@@ -281,7 +281,7 @@ static struct oscap_module XCCDF_GEN_FIX = {
     .help = GEN_OPTS
         "\nFix Options:\n"
 		"   --fix-type <type>             - Fix type. Should be one of: bash, ansible, puppet, anaconda, ignition, kubernetes,\n"
-		"                                   blueprint, kickstart (default: bash).\n"
+		"                                   blueprint, kickstart, bootc (default: bash).\n"
 		"   --output <file>               - Write the script into file.\n"
 		"   --raw                         - Don't write extra headers or boilerplate instructions, only compose the content snippets.\n"
 		"   --result-id <id>              - Fixes will be generated for failed rule-results of the specified TestResult.\n"
@@ -961,10 +961,13 @@ int app_generate_fix(const struct oscap_action *action)
 			remediation_system = "urn:xccdf:fix:script:kubernetes";
 		} else if (strcmp(action->fix_type, "blueprint") == 0) {
 			remediation_system = "urn:redhat:osbuild:blueprint";
+		} else if (strcmp(action->fix_type, "bootc") == 0) {
+			remediation_system = "urn:xccdf:fix:script:bootc";
 		} else {
 			fprintf(stderr,
 					"Unknown fix type '%s'.\n"
-					"Please provide one of: bash, ansible, kickstart, puppet, anaconda, ignition, kubernetes, blueprint.\n",
+					"Please provide one of: bash, ansible, kickstart, puppet, anaconda, ignition, kubernetes, blueprint, bootc.\n"
+					"Or provide a custom template using '--template' instead.\n",
 					action->fix_type);
 			return OSCAP_ERROR;
 		}
@@ -976,6 +979,10 @@ int app_generate_fix(const struct oscap_action *action)
 		fprintf(stderr, "It isn't possible to generate results-oriented Kickstarts.\n");
 		return OSCAP_ERROR;
 	}
+	if (action->id != NULL && action->fix_type != NULL && !strcmp(action->fix_type, "bootc")) {
+		fprintf(stderr, "It isn't possible to generate results-oriented bootc remediations.\n");
+		return OSCAP_ERROR;
+	}
 
 	int ret = OSCAP_ERROR;
 	struct oscap_source *source = oscap_source_new_from_file(action->f_xccdf);
